import time
from xml.etree.ElementTree import SubElement


def set_head(parent, batch_id, pub_date, crossref_config):
    head_tag = SubElement(parent, "head")
    doi_batch_id_tag = SubElement(head_tag, "doi_batch_id")
    doi_batch_id_tag.text = batch_id
    timestamp_tag = SubElement(head_tag, "timestamp")
    timestamp_tag.text = time.strftime("%Y%m%d%H%M%S", pub_date)
    set_depositor(head_tag, crossref_config)
    registrant_tag = SubElement(head_tag, "registrant")
    registrant_tag.text = crossref_config.get("registrant")


def set_depositor(parent, crossref_config):
    depositor_tag = SubElement(parent, "depositor")
    name_tag = SubElement(depositor_tag, "depositor_name")
    name_tag.text = crossref_config.get("depositor_name")
    email_address_tag = SubElement(depositor_tag, "email_address")
    email_address_tag.text = crossref_config.get("email_address")



from xml.etree.ElementTree import SubElement


def do_access_indicators(poa_article, crossref_config):
    return bool(
        crossref_config.get("access_indicators_applies_to") and has_license(poa_article)
    )


def set_access_indicators(parent, poa_article, crossref_config):
    """
    Set the AccessIndicators
    """
    if do_access_indicators(poa_article, crossref_config):
        ai_program_tag = set_ai_program(parent)
        applies_to_list = crossref_config.get("access_indicators_applies_to")
        for applies_to in applies_to_list:
            set_ai_license_ref(ai_program_tag, poa_article.license.href, applies_to)


def has_license(poa_article):
    """check if the article has the minimum requirements of a license"""
    if not poa_article.license:
        return False
    if not poa_article.license.href:
        return False
    return True


def set_ai_program(parent):
    ai_program_tag = SubElement(parent, "ai:program")
    ai_program_tag.set("name", "AccessIndicators")
    return ai_program_tag


def set_ai_license_ref(parent, href, applies_to=None):
    ai_program_ref_tag = SubElement(parent, "ai:license_ref")
    if applies_to:
        ai_program_ref_tag.set("applies_to", applies_to)
    ai_program_ref_tag.text = href



from xml.etree.ElementTree import SubElement
from elifecrossref import access_indicators, clinical_trials, dates, funding


# article types currently supported for depositing simple updates via Crossmark
UPDATES_ARTICLE_TYPES = ["correction", "retraction"]


def do_crossmark(poa_article, crossref_config):
    """check if there are sufficient and correct values to set crossmark data"""
    return bool(
        crossref_config.get("crossmark")
        and (
            crossref_config.get("crossmark_policy")
            or (hasattr(poa_article, "doi") and poa_article.doi)
        )
    )


def set_crossmark(parent, poa_article, crossref_config):
    crossmark = SubElement(parent, "crossmark")

    crossmark_policy = SubElement(crossmark, "crossmark_policy")
    if crossref_config.get("crossmark_policy"):
        crossmark_policy.text = crossref_config.get("crossmark_policy")
    else:
        crossmark_policy.text = poa_article.doi

    if crossref_config.get("crossmark_domains"):
        crossmark_domains = SubElement(crossmark, "crossmark_domains")
        for domain in crossref_config.get("crossmark_domains"):
            crossmark_domain = SubElement(crossmark_domains, "crossmark_domain")
            crossmark_domain_domain = SubElement(crossmark_domain, "domain")
            crossmark_domain_domain.text = domain.get("domain")
            if domain.get("filter"):
                crossmark_domain_filter = SubElement(crossmark_domain, "filter")
                crossmark_domain_filter.text = domain.get("filter")

    if crossref_config.get("crossmark_domain_exclusive"):
        crossmark_domain_exclusive = SubElement(crossmark, "crossmark_domain_exclusive")
        crossmark_domain_exclusive.text = crossref_config.get(
            "crossmark_domain_exclusive"
        )

    if do_updates(poa_article):
        set_updates(crossmark, poa_article, crossref_config)

    set_custom_metadata(crossmark, poa_article, crossref_config)


def do_custom_metadata(poa_article, crossref_config):
    return bool(
        access_indicators.do_access_indicators(poa_article, crossref_config)
        or funding.do_funding(poa_article)
        or clinical_trials.do_clinical_trials(poa_article)
        or do_assertion_tags(poa_article, crossref_config)
    )


def do_assertion_tags(poa_article, crossref_config):
    "decide whether assertion tags should be added to the custom_metadata tag"
    if not poa_article.display_channel or not crossref_config.get(
        "assertion_display_channel_types"
    ):
        return False
    # convert list to lower case values
    display_channel_matches = [
        display_channel.lower()
        for display_channel in crossref_config.get("assertion_display_channel_types")
        if display_channel
    ]
    return bool(poa_article.display_channel.lower() in display_channel_matches)


def set_custom_metadata(parent, poa_article, crossref_config):
    if do_custom_metadata(poa_article, crossref_config):
        custom_metadata = SubElement(parent, "custom_metadata")
        if do_assertion_tags(poa_article, crossref_config):
            set_assertions(custom_metadata, poa_article, crossref_config)
        funding.set_fundref(custom_metadata, poa_article)
        access_indicators.set_access_indicators(
            custom_metadata, poa_article, crossref_config
        )
        clinical_trials.set_clinical_trials(
            custom_metadata, poa_article, crossref_config
        )


def do_updates(poa_article):
    """decide if crossmark updates tag can be added"""
    return bool(
        poa_article.article_type in UPDATES_ARTICLE_TYPES
        and poa_article.related_articles
        and poa_article.related_articles[0].xlink_href
    )


def set_updates(parent, poa_article, crossref_config):
    default_pub_date = None

    updates = SubElement(parent, "updates")
    if poa_article.article_type in UPDATES_ARTICLE_TYPES:
        set_update(
            updates,
            poa_article.article_type,
            dates.iso_date_string(
                dates.get_pub_date(poa_article, crossref_config, default_pub_date)
            ),
            poa_article.related_articles[0].xlink_href,
        )


def set_update(parent, article_type, date, doi):
    update = SubElement(parent, "update")
    update.set("date", date)
    update.set("type", article_type)
    update.text = doi


def set_assertions(parent, poa_article, crossref_config):
    tag_details = []
    tag_details.append(
        {
            "name": "peer_review_transparency",
            "label": "Peer review transparency",
            "group_name": "peer_review_taxonomy",
            "text": "single anonymised",
        }
    )
    tag_details.append(
        {
            "name": "peer_review_interaction",
            "label": "Peer review interaction",
            "group_name": "peer_review_taxonomy",
            "text": "other reviewer(s), editor",
        }
    )
    tag_details.append(
        {
            "name": "peer_review_published",
            "label": "Peer review published",
            "group_name": "peer_review_taxonomy",
            "text": (
                "review summaries, review reports, author/editor communication, "
                "reviewer identities reviewer opt in, editor identities"
            ),
        }
    )
    tag_details.append(
        {
            "name": "post_publication_commenting",
            "label": "Post publication commenting",
            "group_name": "post_publication_commenting",
            "text": "open (sign in with ORCID iD required)",
        }
    )

    for details in tag_details:
        assertion_tag = SubElement(parent, "assertion")
        for attribute in ["group_name", "label", "name"]:
            assertion_tag.set(attribute, details.get(attribute))
        assertion_tag.text = details.get("text")



def elife_style_article_attributes(obj):
    # Special logic for elife article style
    version = ""
    if obj.version:
        version = "-v" + str(obj.version)
    return version


def elife_style_component_attributes(obj):
    # Some special additional logic for elife style
    component_id = obj.id
    if obj.type and obj.type == "abstract":
        if obj.title and "digest" in obj.title.lower():
            component_id = "digest"
        else:
            component_id = "abstract"
    # Set the URL prefix for some types
    prefix1 = ""
    if (
        obj.asset
        and obj.asset in ["figsupp", "data"]
        or obj.type
        and obj.type in ["supplementary-material"]
    ):
        prefix1 = "/figures"
    return component_id, prefix1



from xml.etree.ElementTree import SubElement
from elifecrossref import journal, peer_review, pending_publication


def set_body(parent, poa_articles, crossref_config, default_pub_date, submission_type):
    body_tag = SubElement(parent, "body")

    for poa_article in poa_articles:
        if submission_type == "journal":
            # Create a new journal record for each article
            journal.set_journal(
                body_tag, poa_article, crossref_config, default_pub_date
            )
        elif submission_type == "peer_review":
            peer_review.set_peer_review(body_tag, poa_article, crossref_config)
        elif submission_type == "pending_publication":
            pending_publication.set_pending_publication(
                body_tag, poa_article, crossref_config
            )



from xml.etree.ElementTree import SubElement


def do_funding(poa_article):
    return bool(poa_article.funding_awards)


def set_fundref(parent, poa_article):
    """
    Set the fundref data from the article funding_awards list
    """
    if do_funding(poa_article):
        fr_program_tag = SubElement(parent, "fr:program")
        fr_program_tag.set("name", "fundref")
    for award in poa_article.funding_awards:
        set_funding_award(fr_program_tag, award)


def set_funding_award(parent, award):
    fr_fundgroup_tag = SubElement(parent, "fr:assertion")
    fr_fundgroup_tag.set("name", "fundgroup")
    fr_funder_name_tag = set_funder_name(fr_fundgroup_tag, award)
    if fr_funder_name_tag is not None:
        set_funder_identifier(fr_funder_name_tag, award)
    set_award_number(fr_fundgroup_tag, award)


def set_funder_name(parent, award):
    fr_funder_name_tag = None
    if award.get_funder_name():
        fr_funder_name_tag = SubElement(parent, "fr:assertion")
        fr_funder_name_tag.set("name", "funder_name")
        fr_funder_name_tag.text = award.get_funder_name()
    return fr_funder_name_tag


def set_funder_identifier(parent, award):
    if award.get_funder_name() and award.institution_id:
        fr_funder_identifier_tag = SubElement(parent, "fr:assertion")
        fr_funder_identifier_tag.set("name", "funder_identifier")
        fr_funder_identifier_tag.text = award.institution_id


def set_award_number(parent, award):
    for award_id in award.award_ids:
        fr_award_number_tag = SubElement(parent, "fr:assertion")
        fr_award_number_tag.set("name", "award_number")
        fr_award_number_tag.text = award_id



from xml.etree.ElementTree import SubElement
from elifecrossref import contributor, dates, doi, related, title, access_indicators


def set_peer_review(parent, poa_article, crossref_config):
    for review_article in poa_article.review_articles:
        # Add peer_review for each review
        peer_review_tag = SubElement(parent, "peer_review")
        set_stage(peer_review_tag)
        set_type(peer_review_tag, review_article)

        if review_article.contributors:
            contributor.set_contributors(peer_review_tag, review_article.contributors)

        set_title(peer_review_tag, review_article, poa_article, crossref_config)

        set_review_date(peer_review_tag, review_article.get_date("review_date"))

        # set access indicators
        if review_article.license and review_article.license.href:
            ai_program_tag = access_indicators.set_ai_program(peer_review_tag)
            access_indicators.set_ai_license_ref(
                ai_program_tag, review_article.license.href
            )

        if review_article.related_articles:
            # set the related article DOI to the first of the related_articles doi
            related_article_doi = review_article.related_articles[0].doi
            relations_program_tag = related.set_relations_program(peer_review_tag, None)
            related_item_tag = SubElement(relations_program_tag, "rel:related_item")
            related.set_related_item_work_relation(
                related_item_tag,
                "inter_work_relation",
                "isReviewOf",
                "doi",
                related_article_doi,
            )
            for related_object in review_article.related_objects:
                link_type = related_object.link_type
                if related_object.link_type == "continued-by":
                    link_type = "hasRelatedMaterial"
                related_item_tag = SubElement(relations_program_tag, "rel:related_item")
                related.set_related_item_work_relation(
                    related_item_tag,
                    "inter_work_relation",
                    link_type,
                    "uri",
                    related_object.xlink_href,
                )

        doi.set_doi_data(
            peer_review_tag,
            review_article,
            poa_article,
            crossref_config,
            "peer_review_doi_pattern",
        )


def set_stage(parent):
    parent.set("stage", "pre-publication")


def set_type(parent, review_article):
    if review_article.article_type in [
        "article-commentary",
        "editor-report",
        "decision-letter",
    ]:
        parent.set("type", "editor-report")
    elif review_article.article_type in ["author-comment", "reply"]:
        parent.set("type", "author-comment")


def set_title(parent, review_article, poa_article, crossref_config):
    """concatenate the review and parent article titles"""
    title_value = ": ".join(
        [value for value in [review_article.title, poa_article.title] if value]
    )
    title.set_titles(parent, title_value, crossref_config)


def set_review_date(parent, article_date):
    # article_date is an ArticleDate object
    if article_date:
        date_tag = SubElement(parent, "review_date")
        dates.set_date_detail(date_tag, article_date.date)



from xml.etree.ElementTree import SubElement


def set_article_contributors(parent, poa_article, contrib_types=None):
    # First check for any contributors
    if not poa_article.contributors:
        return
    article_contributors = []
    for contributor in poa_article.contributors:
        if contrib_types:
            # Filter by contrib_type if supplied
            if contributor.contrib_type not in contrib_types:
                continue
        article_contributors.append(contributor)
    set_contributors(parent, article_contributors)


def set_contributors(parent, contributors):
    # If contrib_type is None, all contributors will be added regardless of their type
    contributors_tag = SubElement(parent, "contributors")

    # Ready to add to XML
    # Use the natural list order of contributors when setting the first author
    sequence = "first"
    prev_equal_contrib = None
    for contributor in contributors:
        if (sequence == "first" and prev_equal_contrib is False) or (
            sequence == "first"
            and prev_equal_contrib is True
            and not contributor.equal_contrib
        ):
            # Reset sequence value unless the first contributor and
            # next contributor is also equal_contrib
            sequence = "additional"

        set_contributor(contributors_tag, contributor, sequence)

        # set value for the next loop interation
        prev_equal_contrib = contributor.equal_contrib


def set_contributor(parent, contributor, sequence):
    """add tags for a contributor to the parent tag"""
    if contributor.contrib_type == "on-behalf-of":
        contributor_role = "author"
    else:
        contributor_role = contributor.contrib_type

    # Skip contributors with no surname
    if contributor.surname == "" or contributor.surname is None:
        # Most likely a group author
        set_org_contributor(parent, contributor, contributor_role, sequence)

    else:
        set_person_contributor(parent, contributor, contributor_role, sequence)


def set_org_contributor(parent, contributor, contributor_role, sequence):
    if contributor.collab:
        organization_tag = SubElement(parent, "organization")
        organization_tag.text = contributor.collab
        organization_tag.set("contributor_role", contributor_role)
        organization_tag.set("sequence", sequence)


def set_person_contributor(parent, contributor, contributor_role, sequence):
    person_name_tag = set_person_name(parent, contributor, contributor_role, sequence)
    set_affiliations(person_name_tag, contributor)
    set_orcid(person_name_tag, contributor)


def set_person_name(parent, contributor, contributor_role, sequence):
    person_name_tag = SubElement(parent, "person_name")
    person_name_tag.set("contributor_role", contributor_role)
    person_name_tag.set("sequence", sequence)

    if contributor.given_name:
        given_name_tag = SubElement(person_name_tag, "given_name")
        given_name_tag.text = contributor.given_name

    surname_tag = SubElement(person_name_tag, "surname")
    surname_tag.text = contributor.surname

    if contributor.suffix:
        suffix_tag = SubElement(person_name_tag, "suffix")
        suffix_tag.text = contributor.suffix

    return person_name_tag


def set_orcid(parent, contributor):
    if contributor.orcid:
        orcid_tag = SubElement(parent, "ORCID")
        if hasattr(contributor, "orcid_authenticated"):
            if contributor.orcid_authenticated is True:
                orcid_tag.set("authenticated", "true")
            elif contributor.orcid_authenticated is False:
                orcid_tag.set("authenticated", "false")
        orcid_tag.text = contributor.orcid


def set_affiliations(parent, contributor):
    # Crossref schema limits the number of affilations an author can have
    max_affiliations = 5
    non_blank_affiliations = [
        aff for aff in contributor.affiliations if affiliation_name_place(aff)[0]
    ]
    affiliations_to_add = non_blank_affiliations[0:max_affiliations]
    if affiliations_to_add:
        affiliations_tag = SubElement(parent, "affiliations")
    for aff in affiliations_to_add:
        # format name and place tag strings
        text_name, text_place = affiliation_name_place(aff)
        # add tags
        institution_tag = SubElement(affiliations_tag, "institution")
        institution_name_tag = SubElement(institution_tag, "institution_name")
        institution_name_tag.text = text_name
        if hasattr(aff, "ror") and aff.ror:
            institution_id_tag = SubElement(institution_tag, "institution_id")
            institution_id_tag.set("type", "ror")
            institution_id_tag.text = aff.ror
        if text_place:
            institution_place_tag = SubElement(institution_tag, "institution_place")
            institution_place_tag.text = text_place


def affiliation_name_place(aff):
    "from an Affiliation object, set name and place values to be used in institution tags"
    text_name = ", ".join(
        [value for value in [aff.department, aff.institution] if value]
    )
    text_place = ", ".join([value for value in [aff.city, aff.country] if value])
    if not text_name and aff.text:
        text_name = aff.text
    elif not text_name and text_place:
        # use the place value as the name to satisfy fields required in the Crossref schema
        text_name = text_place
        text_place = ""
    return text_name, text_place



def crossref_mime_type(jats_mime_type):
    """
    Dictionary of lower case JATS mime type to crossRef schema mime type
    """
    mime_types = {}

    mime_types["application/eps"] = "application/eps"
    mime_types["application/gz"] = "application/gzip"
    mime_types["application/tar.gz"] = "application/gzip"
    mime_types["application/doc"] = "application/msword"
    mime_types["application/pdf"] = "application/pdf"
    mime_types["application/rtf"] = "application/rtf"
    mime_types["application/xls"] = "application/vnd.ms-excel"
    mime_types[
        "application/pptx"
    ] = "application/vnd.openxmlformats-officedocument.presentationml.presentation"
    mime_types[
        "application/xlsx"
    ] = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
    mime_types[
        "application/docx"
    ] = "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
    mime_types["application/xml"] = "application/xml"

    mime_types["chemical/pdb"] = "chemical/x-pdb"

    mime_types["image/tif"] = "image/tiff"
    mime_types["image/tiff"] = "image/tiff"
    mime_types["application/png"] = "image/png"

    mime_types["text/plain"] = "text/plain"
    mime_types["application/csv"] = "text/plain"
    mime_types["application/txt"] = "text/plain"
    mime_types["text/pl"] = "text/plain"
    mime_types["text/py"] = "text/plain"
    mime_types["text/txt"] = "text/plain"
    mime_types["text/rtf"] = "text/rtf"

    mime_types["video/avi"] = "video/avi"
    mime_types["video/mp4"] = "video/mp4"
    mime_types["video/mpeg"] = "video/mpeg"
    mime_types["video/mpg"] = "video/mpeg"
    mime_types["video/mov"] = "video/quicktime"
    mime_types["video/wmv"] = "video/x-ms-wmv"
    mime_types["video/gif"] = "image/gif"

    return mime_types.get(jats_mime_type.lower())
